/*
 * BitVector.h
 *
 * Created on: Feb 2, 2009
 * \author jonathanmasci
 */

#ifndef BITVECTOR_H_
#define BITVECTOR_H_

typedef unsigned char uchar;
typedef unsigned int uint;

#define _BIT_WORD_SIZE (sizeof(uchar) * 8)

class BitVector
{
public:
  struct _bit_reference
  {
    uint _mask;
    uchar *_byte;

    _bit_reference() : _mask(0), _byte(0) { }
    _bit_reference(uchar *byte, uint mask) : _mask(mask), _byte(byte) { }

    operator bool() const
    {
      return !!(*_byte & _mask); // double-bang trick
    }

    _bit_reference& operator=(bool x)
    {
      if (x)
        *_byte |= _mask;
      else
        *_byte &= ~_mask;
      return *this;
    }

    _bit_reference& operator=(const _bit_reference& x)
    {
      return *this = x;
    }

    bool operator==(const _bit_reference& x) const
    {
      return bool(*this) == bool(x);
    }

    bool operator<(const _bit_reference& x) const {
      return !bool(*this) && bool(x);
    }
  };

  BitVector(uint length);
  BitVector(uchar *buffer, uint length, uint bits);
  virtual ~BitVector();
  void
  set(uint pos);
  void
  unset(uint pos);
  bool
  get(uint pos);
  void
  flip(uint pos);
  uint
  size();
  uchar *
  toString();
  void
  clear();
  uchar *
  getBitContainer();

  _bit_reference& operator[](uint pos);
  const _bit_reference& operator[](uint pos) const;

protected:
  inline const uint
  getByte(uint pos) const;
  inline const uint
  getBitInByte(uint pos, uint byte) const;
  inline const bool
  checkRange(uint pos) const;

  bool _to_delete_buffer;
  uchar *_buffer;
  uint _buffer_size; // in bytes, sizeof(uchar)
  uint _size; // in bits
};

#endif /* BITVECTOR_H_ */
